@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# Copyright (c) 2002-2020 "Neo4j,"
+# Copyright (c) "Neo4j"
 # Neo4j Sweden AB [http://neo4j.com]
 #
 # This file is part of Neo4j.
@@ -266,7 +266,10 @@ class Clock:
 
         :raises OverflowError:
         """
-        return ClockTime(-int(mktime(gmtime(0))))
+        # Adding and subtracting two days to avoid passing a pre-epoch time to
+        # `mktime`, which can cause a `OverflowError` on some platforms (e.g.,
+        # Windows).
+        return ClockTime(-int(mktime(gmtime(172800))) + 172800)
 
     def local_time(self):
         """ Read and return the current local time from this clock, measured relative to the Unix Epoch.
@@ -389,6 +392,12 @@ class Duration(tuple):
     def __str__(self):
         return self.iso_format()
 
+    def __copy__(self):
+        return self.__new(self.ticks, self.hour, self.minute, self.second, self.tzinfo)
+
+    def __deepcopy__(self, memodict={}):
+        return self.__copy__()
+
     @classmethod
     def from_iso_format(cls, s):
         m = DURATION_ISO_PATTERN.match(s)
@@ -845,6 +854,12 @@ class Date(metaclass=DateType):
         except TypeError:
             return NotImplemented
 
+    def __copy__(self):
+        return self.__new(self.__ordinal, self.__year, self.__month, self.__day)
+
+    def __deepcopy__(self, *args, **kwargs):
+        return self.__copy__()
+
     # INSTANCE METHODS #
 
     def replace(self, **kwargs):
@@ -1133,6 +1148,12 @@ class Time(metaclass=TimeType):
     def __sub__(self, other):
         return NotImplemented
 
+    def __copy__(self):
+        return self.__new(self.__ticks, self.__hour, self.__minute, self.__second, self.__tzinfo)
+
+    def __deepcopy__(self, *args, **kwargs):
+        return self.__copy__()
+
     # INSTANCE METHODS #
 
     def replace(self, **kwargs):
@@ -1474,6 +1495,12 @@ class DateTime(metaclass=DateTimeType):
             return self.__add__(-other)
         return NotImplemented
 
+    def __copy__(self):
+        return self.combine(self.__date, self.__time)
+
+    def __deepcopy__(self, *args, **kwargs):
+        return self.__copy__()
+
     # INSTANCE METHODS #
 
     def date(self):
